@@ -27,10 +27,6 @@
 
 # include <config.h>
 
-# if HAVE_POLKIT0
-#  include <dbus/dbus.h>
-# endif
-
 # include <rpc/types.h>
 # include <rpc/xdr.h>
 # include "remote_protocol.h"
@@ -38,7 +34,9 @@
 # include "logging.h"
 # include "threads.h"
 # include "network.h"
-# include "virnetsaslcontext.h"
+# if HAVE_SASL
+#  include "virnetsaslcontext.h"
+# endif
 # include "virnetserverprogram.h"
 
 # if WITH_DTRACE
@@ -70,7 +68,9 @@ struct daemonClientPrivate {
 
     int domainEventCallbackID[VIR_DOMAIN_EVENT_ID_LAST];
 
+# if HAVE_SASL
     virNetSASLSessionPtr sasl;
+# endif
 
     /* This is only valid if a remote open call has been made on this
      * connection, otherwise it will be NULL.  Also if remote close is
@@ -81,34 +81,10 @@ struct daemonClientPrivate {
     daemonClientStreamPtr streams;
 };
 
+# if HAVE_SASL
 extern virNetSASLContextPtr saslCtxt;
+# endif
 extern virNetServerProgramPtr remoteProgram;
 extern virNetServerProgramPtr qemuProgram;
 
-/* Main server state */
-struct qemud_server {
-    int privileged;
-
-    int sigread;
-    int sigwrite;
-    char *logDir;
-    pthread_t eventThread;
-    unsigned int hasEventThread :1;
-    unsigned int quitEventThread :1;
-# ifdef HAVE_AVAHI
-    struct libvirtd_mdns *mdns;
-# endif
-# if HAVE_SASL
-    char **saslUsernameWhitelist;
-# endif
-# if HAVE_POLKIT0
-    DBusConnection *sysbus;
-# endif
-};
-
-
-# if HAVE_POLKIT
-int qemudGetSocketIdentity(int fd, uid_t *uid, pid_t *pid);
-# endif
-
 #endif
