@@ -104,6 +104,15 @@ daemonStreamMessageFinished(virNetMessagePtr msg,
     daemonStreamUpdateEvents(stream);
 }
 
+
+static void
+daemonStreamEventFreeFunc(void *opaque)
+{
+    virNetServerClientPtr client = opaque;
+
+    virNetServerClientFree(client);
+}
+
 /*
  * Callback that gets invoked when a stream becomes writable/readable
  */
@@ -329,7 +338,7 @@ int daemonFreeClientStream(virNetServerClientPtr client,
         memset(msg, 0, sizeof(*msg));
         msg->header.type = VIR_NET_REPLY;
         if (virNetServerClientSendMessage(client, msg) < 0) {
-            virNetServerClientMarkClose(client);
+            virNetServerClientImmediateClose(client);
             virNetMessageFree(msg);
             ret = -1;
         }
@@ -361,9 +370,11 @@ int daemonAddClientStream(virNetServerClientPtr client,
     }
 
     if (virStreamEventAddCallback(stream->st, 0,
-                                  daemonStreamEvent, client, NULL) < 0)
+                                  daemonStreamEvent, client,
+                                  daemonStreamEventFreeFunc) < 0)
         return -1;
 
+    virNetServerClientRef(client);
     if ((stream->filterID = virNetServerClientAddFilter(client,
                                                         daemonStreamFilter,
                                                         stream)) < 0) {
@@ -597,7 +608,7 @@ daemonStreamHandleWrite(virNetServerClientPtr client,
         virNetMessageQueueServe(&stream->rx);
         if (ret < 0) {
             virNetMessageFree(msg);
-            virNetServerClientMarkClose(client);
+            virNetServerClientImmediateClose(client);
             return -1;
         }
 
@@ -612,7 +623,7 @@ daemonStreamHandleWrite(virNetServerClientPtr client,
             msg->header.type = VIR_NET_REPLY;
             if (virNetServerClientSendMessage(client, msg) < 0) {
                 virNetMessageFree(msg);
-                virNetServerClientMarkClose(client);
+                virNetServerClientImmediateClose(client);
                 return -1;
             }
         }
