@@ -1,7 +1,7 @@
 /*
  * cpu_conf.h: CPU XML handling
  *
- * Copyright (C) 2009, 2010 Red Hat, Inc.
+ * Copyright (C) 2009-2011 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -307,7 +307,7 @@ error:
 char *
 virCPUDefFormat(virCPUDefPtr def,
                 const char *indent,
-                int flags)
+                unsigned int flags)
 {
     virBuffer buf = VIR_BUFFER_INITIALIZER;
 
@@ -331,7 +331,7 @@ int
 virCPUDefFormatBuf(virBufferPtr buf,
                    virCPUDefPtr def,
                    const char *indent,
-                   int flags)
+                   unsigned int flags)
 {
     unsigned int i;
 
