@@ -35,7 +35,7 @@
 #include "xml.h"
 #include "virterror_internal.h"
 #include "uuid.h"
-#include "files.h"
+#include "virfile.h"
 
 #define VIR_FROM_THIS VIR_FROM_STORAGE
 
