@@ -715,7 +715,7 @@ virNWFilterInstantiate(virConnectPtr conn,
         if (teardownOld && rc == 0)
             techdriver->tearOldRules(conn, ifname);
 
-        if (rc == 0 && ifaceCheck(false, ifname, NULL, ifindex)) {
+        if (rc == 0 && (ifaceCheck(false, ifname, NULL, ifindex) < 0)) {
             /* interface changed/disppeared */
             techdriver->allTeardown(ifname);
             rc = 1;
@@ -898,7 +898,7 @@ _virNWFilterInstantiateFilter(virConnectPtr conn,
     int ifindex;
     int rc;
 
-    if (ifaceGetIndex(true, net->ifname, &ifindex))
+    if (ifaceGetIndex(true, net->ifname, &ifindex) < 0)
         return 1;
 
     virNWFilterLockFilterUpdates();
@@ -954,8 +954,8 @@ virNWFilterInstantiateFilterLate(virConnectPtr conn,
                                         &foundNewFilter);
     if (rc) {
         /* something went wrong... 'DOWN' the interface */
-        if (ifaceCheck(false, ifname, NULL, ifindex) != 0 ||
-            ifaceDown(ifname)) {
+        if ((ifaceCheck(false, ifname, NULL, ifindex) < 0) ||
+            (ifaceDown(ifname) < 0)) {
             /* assuming interface disappeared... */
             _virNWFilterTeardownFilter(ifname);
         }
