@@ -1,7 +1,7 @@
 /*
  * qemu_conf.h: QEMU configuration management
  *
- * Copyright (C) 2006-2007, 2009-2010 Red Hat, Inc.
+ * Copyright (C) 2006-2007, 2009-2011 Red Hat, Inc.
  * Copyright (C) 2006 Daniel P. Berrange
  *
  * This library is free software; you can redistribute it and/or
@@ -119,6 +119,9 @@ struct qemud_driver {
     char *dumpImageFormat;
 
     char *autoDumpPath;
+    bool autoDumpBypassCache;
+
+    bool autoStartBypassCache;
 
     pciDeviceList *activePciHostdevs;
 
