@@ -9,8 +9,8 @@
 # for both remote_protocol.x and qemu_protocol.x, you would run the
 # following:
 #
-# remote_generator.pl -c -t remote ../src/remote/remote_protocol.x
-# remote_generator.pl -t qemu ../src/remote/qemu_protocol.x
+# gendispatch.pl -t remote ../src/remote/remote_protocol.x
+# gendispatch.pl -t qemu ../src/remote/qemu_protocol.x
 #
 # By Richard Jones <rjones@redhat.com>
 # Extended by Matthias Bolte <matthias.bolte@googlemail.com>
@@ -20,8 +20,8 @@ use strict;
 use Getopt::Std;
 
 # Command line options.
-our ($opt_p, $opt_t, $opt_a, $opt_r, $opt_d, $opt_c, $opt_b, $opt_k);
-getopts ('ptardcbk');
+our ($opt_p, $opt_t, $opt_a, $opt_r, $opt_d, $opt_b, $opt_k);
+getopts ('ptardbk');
 
 my $structprefix = shift or die "missing prefix argument";
 my $protocol = shift or die "missing protocol argument";
@@ -45,18 +45,6 @@ sub name_to_ProcName {
 # opinion about the name, args and return type of each RPC.
 my ($name, $ProcName, $id, $flags, %calls, @calls);
 
-# only generate a close method if -c was passed
-if ($opt_c) {
-    # REMOTE_PROC_CLOSE has no args or ret.
-    $calls{close} = {
-        name => "close",
-        ProcName => "Close",
-        UC_NAME => "CLOSE",
-        args => "void",
-        ret => "void",
-    };
-}
-
 my $collect_args_members = 0;
 my $collect_ret_members = 0;
 my $last_name;
@@ -143,6 +131,20 @@ while (<PROTOCOL>) {
         $flags = $3;
         $ProcName = name_to_ProcName ($name);
 
+        if (!exists $calls{$name}) {
+            # that the argument and return value cases have not yet added
+            # this procedure to the calls hash means that it has no arguments
+            # and no return value. add it to the calls hash now because all
+            # procedures have to be listed in the calls hash
+            $calls{$name} = {
+                name => $name,
+                ProcName => $ProcName,
+                UC_NAME => uc $name,
+                args => "void",
+                ret => "void"
+            }
+        }
+
         if ($opt_b or $opt_k) {
             if (!($flags =~ m/^\s*\/\*\s*(\S+)\s+(\S+)\s*(.*)\*\/\s*$/)) {
                 die "invalid generator flags for ${procprefix}_PROC_${name}"
@@ -218,6 +220,8 @@ my $long_legacy = {
     GetLibVersion               => { ret => { lib_ver => 1 } },
     GetVersion                  => { ret => { hv_ver => 1 } },
     NodeGetInfo                 => { ret => { memory => 1 } },
+    DomainBlockPull             => { arg => { bandwidth => 1 } },
+    DomainBlockJobSetSpeed      => { arg => { bandwidth => 1 } },
 };
 
 sub hyper_to_long
@@ -239,7 +243,7 @@ sub hyper_to_long
 # Output
 
 print <<__EOF__;
-/* Automatically generated by remote_generator.pl.
+/* Automatically generated by gendispatch.pl.
  * Do not edit this file.  Any changes you make will be lost.
  */
 __EOF__
@@ -821,6 +825,8 @@ elsif ($opt_b) {
                 $proc_name = "ConnectBaselineCPU"
             } elsif ($call->{ProcName} eq "CPUCompare") {
                 $proc_name = "ConnectCompareCPU"
+            } elsif ($structprefix eq "qemu" && $call->{ProcName} =~ /^Domain/) {
+                $proc_name =~ s/^(Domain)/${1}Qemu/;
             }
 
             if ($single_ret_as_list) {
@@ -973,6 +979,7 @@ elsif ($opt_k) {
         my @args_check_list = ();
         my @setters_list = ();
         my @setters_list2 = ();
+        my @free_list = ();
         my $priv_src = "conn";
         my $priv_name = "privateData";
         my $call_args = "&args";
@@ -1099,6 +1106,7 @@ elsif ($opt_k) {
                                          "        xdr_free((xdrproc_t)xdr_$call->{args}, (char *)&args);\n" .
                                          "        goto done;\n" .
                                          "    }");
+                    push(@free_list, "    remoteFreeTypedParameters(args.params.params_val, args.params.params_len);\n");
                 } elsif ($args_member =~ m/^((?:unsigned )?int) (\S+);\s*\/\*\s*call-by-reference\s*\*\//) {
                     my $type_name = "$1 *";
                     my $arg_name = $2;
@@ -1323,7 +1331,7 @@ elsif ($opt_k) {
         # print function
         print "\n";
         print "static $single_ret_type\n";
-        print "remote$call->{ProcName}(";
+        print "$structprefix$call->{ProcName}(";
 
         print join(", ", @args_list);
 
@@ -1350,11 +1358,11 @@ elsif ($opt_k) {
         if ($call->{streamflag} ne "none") {
             print "\n";
             print "    if (!(netst = virNetClientStreamNew(priv->remoteProgram, REMOTE_PROC_$call->{UC_NAME}, priv->counter)))\n";
-            print "       goto done;\n";
+            print "        goto done;\n";
             print "\n";
-            print "    if (virNetClientAddStream(priv->client, netst) < 0) {";
-            print "       virNetClientStreamFree(netst);\n";
-            print "       goto done;\n";
+            print "    if (virNetClientAddStream(priv->client, netst) < 0) {\n";
+            print "        virNetClientStreamFree(netst);\n";
+            print "        goto done;\n";
             print "    }";
             print "\n";
             print "    st->driver = &remoteStreamDrv;\n";
@@ -1417,8 +1425,13 @@ elsif ($opt_k) {
             print "    memset(&ret, 0, sizeof ret);\n";
         }
 
+        my $callflags = "0";
+        if ($structprefix eq "qemu") {
+            $callflags = "REMOTE_CALL_QEMU";
+        }
+
         print "\n";
-        print "    if (call($priv_src, priv, 0, ${procprefix}_PROC_$call->{UC_NAME},\n";
+        print "    if (call($priv_src, priv, $callflags, ${procprefix}_PROC_$call->{UC_NAME},\n";
         print "             (xdrproc_t)xdr_$argtype, (char *)$call_args,\n";
         print "             (xdrproc_t)xdr_$rettype, (char *)$call_ret) == -1) {\n";
 
@@ -1489,6 +1502,9 @@ elsif ($opt_k) {
 
         print "\n";
         print "done:\n";
+
+        print join("\n", @free_list);
+
         print "    remoteDriverUnlock(priv);\n";
         print "    return rv;\n";
         print "}\n";
