@@ -34,11 +34,14 @@
 #include "threads.h"
 #include "threadpool.h"
 #include "util.h"
-#include "files.h"
+#include "virfile.h"
 #include "event.h"
 #if HAVE_AVAHI
 # include "virnetservermdns.h"
 #endif
+#if HAVE_DBUS
+# include <dbus/dbus.h>
+#endif
 
 #define VIR_FROM_THIS VIR_FROM_RPC
 #define virNetError(code, ...)                                    \
@@ -84,6 +87,10 @@ struct _virNetServer {
     virNetServerMDNSGroupPtr mdnsGroup;
 #endif
 
+#if HAVE_DBUS
+    DBusConnection *sysbus;
+#endif
+
     size_t nservices;
     virNetServerServicePtr *services;
 
@@ -124,8 +131,6 @@ static void virNetServerHandleJob(void *jobOpaque, void *opaque)
     virNetServerProgramPtr prog = NULL;
     size_t i;
 
-    virNetServerClientRef(job->client);
-
     virNetServerLock(srv);
     VIR_DEBUG("server=%p client=%p message=%p",
               srv, job->client, job->msg);
@@ -257,8 +262,9 @@ static void virNetServerFatalSignal(int sig, siginfo_t *siginfo ATTRIBUTE_UNUSED
 #ifdef SIGUSR2
     if (sig != SIGUSR2) {
 #endif
-        sig_action.sa_handler = SIG_IGN;
+        sig_action.sa_handler = SIG_DFL;
         sigaction(sig, &sig_action, NULL);
+        raise(sig);
 #ifdef SIGUSR2
     }
 #endif
@@ -270,6 +276,7 @@ virNetServerPtr virNetServerNew(size_t min_workers,
                                 size_t max_workers,
                                 size_t max_clients,
                                 const char *mdnsGroupName,
+                                bool connectDBus ATTRIBUTE_UNUSED,
                                 virNetServerClientInitHook clientInitHook)
 {
     virNetServerPtr srv;
@@ -301,11 +308,31 @@ virNetServerPtr virNetServerNew(size_t min_workers,
     if (srv->mdnsGroupName) {
         if (!(srv->mdns = virNetServerMDNSNew()))
             goto error;
-        if (!(srv->mdnsGroup = virNetServerMDNSAddGroup(srv->mdns, mdnsGroupName)))
+        if (!(srv->mdnsGroup = virNetServerMDNSAddGroup(srv->mdns,
+                                                        srv->mdnsGroupName)))
             goto error;
     }
 #endif
 
+#if HAVE_DBUS
+    if (connectDBus) {
+        DBusError derr;
+
+        dbus_connection_set_change_sigpipe(FALSE);
+        dbus_threads_init_default();
+
+        dbus_error_init(&derr);
+        srv->sysbus = dbus_bus_get(DBUS_BUS_SYSTEM, &derr);
+        if (!(srv->sysbus)) {
+            VIR_ERROR(_("Failed to connect to system bus for PolicyKit auth: %s"),
+                      derr.message);
+            dbus_error_free(&derr);
+            goto error;
+        }
+        dbus_connection_set_exit_on_disconnect(srv->sysbus, FALSE);
+    }
+#endif
+
     if (virMutexInit(&srv->lock) < 0) {
         virNetError(VIR_ERR_INTERNAL_ERROR, "%s",
                     _("cannot initialize mutex"));
@@ -363,6 +390,14 @@ bool virNetServerIsPrivileged(virNetServerPtr srv)
 }
 
 
+#if HAVE_DBUS
+DBusConnection* virNetServerGetDBusConn(virNetServerPtr srv)
+{
+    return srv->sysbus;
+}
+#endif
+
+
 void virNetServerAutoShutdown(virNetServerPtr srv,
                               unsigned int timeout,
                               virNetServerAutoShutdownFunc func,
@@ -377,7 +412,6 @@ void virNetServerAutoShutdown(virNetServerPtr srv,
     virNetServerUnlock(srv);
 }
 
-
 static sig_atomic_t sigErrors = 0;
 static int sigLastErrno = 0;
 static int sigWrite = -1;
@@ -438,7 +472,7 @@ cleanup:
 
 static int virNetServerSignalSetup(virNetServerPtr srv)
 {
-    int fds[2];
+    int fds[2] = { -1, -1 };
 
     if (srv->sigwrite != -1)
         return 0;
@@ -636,6 +670,7 @@ void virNetServerRun(virNetServerPtr srv)
         goto cleanup;
     }
 
+    VIR_DEBUG("srv=%p quit=%d", srv, srv->quit);
     while (!srv->quit) {
         /* A shutdown timeout is specified, so check
          * if any drivers have active state, if not
@@ -668,6 +703,9 @@ void virNetServerRun(virNetServerPtr srv)
 
     reprocess:
         for (i = 0 ; i < srv->nclients ; i++) {
+            /* Coverity 5.3.0 couldn't see that srv->clients is non-NULL
+             * if srv->nclients is non-zero.  */
+            sa_assert(srv->clients);
             if (virNetServerClientWantClose(srv->clients[i]))
                 virNetServerClientClose(srv->clients[i]);
             if (virNetServerClientIsClosed(srv->clients[i])) {
@@ -696,6 +734,7 @@ void virNetServerQuit(virNetServerPtr srv)
 {
     virNetServerLock(srv);
 
+    VIR_DEBUG("Quit requested %p", srv);
     srv->quit = 1;
 
     virNetServerUnlock(srv);
@@ -746,6 +785,14 @@ void virNetServerFree(virNetServerPtr srv)
     VIR_FREE(srv->clients);
 
     VIR_FREE(srv->mdnsGroupName);
+#if HAVE_AVAHI
+    virNetServerMDNSFree(srv->mdns);
+#endif
+
+#if HAVE_DBUS
+    if (srv->sysbus)
+        dbus_connection_unref(srv->sysbus);
+#endif
 
     virNetServerUnlock(srv);
     virMutexDestroy(&srv->lock);
