@@ -61,6 +61,7 @@ struct _virNetServerClient
 {
     int refs;
     bool wantClose;
+    bool delayedClose;
     virMutex lock;
     virNetSocketPtr sock;
     int auth;
@@ -155,10 +156,17 @@ virNetServerClientCalculateHandleMode(virNetServerClientPtr client) {
         if (client->tx)
             mode |= VIR_EVENT_HANDLE_WRITABLE;
     }
-    VIR_DEBUG("mode=%d", mode);
+    VIR_DEBUG("mode=%o", mode);
     return mode;
 }
 
+static void virNetServerClientEventFree(void *opaque)
+{
+    virNetServerClientPtr client = opaque;
+
+    virNetServerClientFree(client);
+}
+
 /*
  * @server: a locked or unlocked server object
  * @client: a locked client object
@@ -167,12 +175,17 @@ static int virNetServerClientRegisterEvent(virNetServerClientPtr client)
 {
     int mode = virNetServerClientCalculateHandleMode(client);
 
+    client->refs++;
     VIR_DEBUG("Registering client event callback %d", mode);
-    if (virNetSocketAddIOCallback(client->sock,
+    if (!client->sock ||
+        virNetSocketAddIOCallback(client->sock,
                                   mode,
                                   virNetServerClientDispatchEvent,
-                                  client) < 0)
+                                  client,
+                                  virNetServerClientEventFree) < 0) {
+        client->refs--;
         return -1;
+    }
 
     return 0;
 }
@@ -374,9 +387,10 @@ int virNetServerClientGetTLSKeySize(virNetServerClientPtr client)
 
 int virNetServerClientGetFD(virNetServerClientPtr client)
 {
-    int fd = 0;
+    int fd = -1;
     virNetServerClientLock(client);
-    fd = virNetSocketGetFD(client->sock);
+    if (client->sock)
+        fd = virNetSocketGetFD(client->sock);
     virNetServerClientUnlock(client);
     return fd;
 }
@@ -384,9 +398,10 @@ int virNetServerClientGetFD(virNetServerClientPtr client)
 int virNetServerClientGetLocalIdentity(virNetServerClientPtr client,
                                        uid_t *uid, pid_t *pid)
 {
-    int ret;
+    int ret = -1;
     virNetServerClientLock(client);
-    ret = virNetSocketGetLocalIdentity(client->sock, uid, pid);
+    if (client->sock)
+        ret = virNetSocketGetLocalIdentity(client->sock, uid, pid);
     virNetServerClientUnlock(client);
     return ret;
 }
@@ -401,7 +416,7 @@ bool virNetServerClientIsSecure(virNetServerClientPtr client)
     if (client->sasl)
         secure = true;
 #endif
-    if (virNetSocketIsLocal(client->sock))
+    if (client->sock && virNetSocketIsLocal(client->sock))
         secure = true;
     virNetServerClientUnlock(client);
     return secure;
@@ -490,12 +505,16 @@ void virNetServerClientSetDispatcher(virNetServerClientPtr client,
 
 const char *virNetServerClientLocalAddrString(virNetServerClientPtr client)
 {
+    if (!client->sock)
+        return NULL;
     return virNetSocketLocalAddrString(client->sock);
 }
 
 
 const char *virNetServerClientRemoteAddrString(virNetServerClientPtr client)
 {
+    if (!client->sock)
+        return NULL;
     return virNetSocketRemoteAddrString(client->sock);
 }
 
@@ -558,10 +577,7 @@ void virNetServerClientClose(virNetServerClientPtr client)
         virNetTLSSessionFree(client->tls);
         client->tls = NULL;
     }
-    if (client->sock) {
-        virNetSocketFree(client->sock);
-        client->sock = NULL;
-    }
+    client->wantClose = true;
 
     while (client->rx) {
         virNetMessagePtr msg
@@ -574,6 +590,11 @@ void virNetServerClientClose(virNetServerClientPtr client)
         virNetMessageFree(msg);
     }
 
+    if (client->sock) {
+        virNetSocketFree(client->sock);
+        client->sock = NULL;
+    }
+
     virNetServerClientUnlock(client);
 }
 
@@ -587,7 +608,14 @@ bool virNetServerClientIsClosed(virNetServerClientPtr client)
     return closed;
 }
 
-void virNetServerClientMarkClose(virNetServerClientPtr client)
+void virNetServerClientDelayedClose(virNetServerClientPtr client)
+{
+    virNetServerClientLock(client);
+    client->delayedClose = true;
+    virNetServerClientUnlock(client);
+}
+
+void virNetServerClientImmediateClose(virNetServerClientPtr client)
 {
     virNetServerClientLock(client);
     client->wantClose = true;
@@ -700,8 +728,10 @@ readmore:
 
     /* Either done with length word header */
     if (client->rx->bufferLength == VIR_NET_MESSAGE_LEN_MAX) {
-        if (virNetMessageDecodeLength(client->rx) < 0)
+        if (virNetMessageDecodeLength(client->rx) < 0) {
+            client->wantClose = true;
             return;
+        }
 
         virNetServerClientUpdateEvent(client);
 
@@ -742,10 +772,12 @@ readmore:
 
         /* Send off to for normal dispatch to workers */
         if (msg) {
+            client->refs++;
             if (!client->dispatchFunc ||
                 client->dispatchFunc(client, msg, client->dispatchOpaque) < 0) {
                 virNetMessageFree(msg);
                 client->wantClose = true;
+                client->refs--;
                 return;
             }
         }
@@ -754,9 +786,10 @@ readmore:
         if (client->nrequests < client->nrequests_max) {
             if (!(client->rx = virNetMessageNew())) {
                 client->wantClose = true;
+            } else {
+                client->rx->bufferLength = VIR_NET_MESSAGE_LEN_MAX;
+                client->nrequests++;
             }
-            client->rx->bufferLength = VIR_NET_MESSAGE_LEN_MAX;
-            client->nrequests++;
         }
         virNetServerClientUpdateEvent(client);
     }
@@ -831,7 +864,9 @@ virNetServerClientDispatchWrite(virNetServerClientPtr client)
             /* Get finished msg from head of tx queue */
             msg = virNetMessageQueueServe(&client->tx);
 
-            if (msg->header.type == VIR_NET_REPLY) {
+            if (msg->header.type == VIR_NET_REPLY ||
+                (msg->header.type == VIR_NET_STREAM &&
+                 msg->header.status != VIR_NET_CONTINUE)) {
                 client->nrequests--;
                 /* See if the recv queue is currently throttled */
                 if (!client->rx &&
@@ -848,6 +883,9 @@ virNetServerClientDispatchWrite(virNetServerClientPtr client)
             virNetMessageFree(msg);
 
             virNetServerClientUpdateEvent(client);
+
+            if (client->delayedClose)
+                client->wantClose = true;
          }
     }
 }
