@@ -45,7 +45,7 @@
 #include "memory.h"
 #include "storage_backend.h"
 #include "logging.h"
-#include "files.h"
+#include "virfile.h"
 #include "fdstream.h"
 #include "configmake.h"
 
@@ -322,7 +322,10 @@ storagePoolLookupByVolume(virStorageVolPtr vol) {
 static virDrvOpenStatus
 storageOpen(virConnectPtr conn,
             virConnectAuthPtr auth ATTRIBUTE_UNUSED,
-            int flags ATTRIBUTE_UNUSED) {
+            unsigned int flags)
+{
+    virCheckFlags(VIR_CONNECT_RO, VIR_DRV_OPEN_ERROR);
+
     if (!driverState)
         return VIR_DRV_OPEN_DECLINED;
 
@@ -516,13 +519,16 @@ cleanup:
 static virStoragePoolPtr
 storagePoolCreate(virConnectPtr conn,
                   const char *xml,
-                  unsigned int flags ATTRIBUTE_UNUSED) {
+                  unsigned int flags)
+{
     virStorageDriverStatePtr driver = conn->storagePrivateData;
     virStoragePoolDefPtr def;
     virStoragePoolObjPtr pool = NULL;
     virStoragePoolPtr ret = NULL;
     virStorageBackendPtr backend;
 
+    virCheckFlags(0, NULL);
+
     storageDriverLock(driver);
     if (!(def = virStoragePoolDefParseString(xml)))
         goto cleanup;
@@ -567,12 +573,15 @@ cleanup:
 static virStoragePoolPtr
 storagePoolDefine(virConnectPtr conn,
                   const char *xml,
-                  unsigned int flags ATTRIBUTE_UNUSED) {
+                  unsigned int flags)
+{
     virStorageDriverStatePtr driver = conn->storagePrivateData;
     virStoragePoolDefPtr def;
     virStoragePoolObjPtr pool = NULL;
     virStoragePoolPtr ret = NULL;
 
+    virCheckFlags(0, NULL);
+
     storageDriverLock(driver);
     if (!(def = virStoragePoolDefParseString(xml)))
         goto cleanup;
@@ -657,12 +666,15 @@ cleanup:
 
 static int
 storagePoolStart(virStoragePoolPtr obj,
-                 unsigned int flags ATTRIBUTE_UNUSED) {
+                 unsigned int flags)
+{
     virStorageDriverStatePtr driver = obj->conn->storagePrivateData;
     virStoragePoolObjPtr pool;
     virStorageBackendPtr backend;
     int ret = -1;
 
+    virCheckFlags(0, -1);
+
     storageDriverLock(driver);
     pool = virStoragePoolObjFindByUUID(&driver->pools, obj->uuid);
     storageDriverUnlock(driver);
@@ -848,12 +860,15 @@ cleanup:
 
 static int
 storagePoolRefresh(virStoragePoolPtr obj,
-                   unsigned int flags ATTRIBUTE_UNUSED) {
+                   unsigned int flags)
+{
     virStorageDriverStatePtr driver = obj->conn->storagePrivateData;
     virStoragePoolObjPtr pool;
     virStorageBackendPtr backend;
     int ret = -1;
 
+    virCheckFlags(0, -1);
+
     storageDriverLock(driver);
     pool = virStoragePoolObjFindByUUID(&driver->pools, obj->uuid);
 
@@ -940,11 +955,14 @@ cleanup:
 
 static char *
 storagePoolGetXMLDesc(virStoragePoolPtr obj,
-                      unsigned int flags ATTRIBUTE_UNUSED) {
+                      unsigned int flags)
+{
     virStorageDriverStatePtr driver = obj->conn->storagePrivateData;
     virStoragePoolObjPtr pool;
     char *ret = NULL;
 
+    virCheckFlags(0, NULL);
+
     storageDriverLock(driver);
     pool = virStoragePoolObjFindByUUID(&driver->pools, obj->uuid);
     storageDriverUnlock(driver);
@@ -1019,10 +1037,8 @@ storagePoolSetAutostart(virStoragePoolPtr obj,
 
     if (pool->autostart != autostart) {
         if (autostart) {
-            int err;
-
-            if ((err = virFileMakePath(driver->autostartDir))) {
-                virReportSystemError(err,
+            if (virFileMakePath(driver->autostartDir) < 0) {
+                virReportSystemError(errno,
                                      _("cannot create autostart directory %s"),
                                      driver->autostartDir);
                 goto cleanup;
@@ -1263,13 +1279,16 @@ static int storageVolumeDelete(virStorageVolPtr obj, unsigned int flags);
 static virStorageVolPtr
 storageVolumeCreateXML(virStoragePoolPtr obj,
                        const char *xmldesc,
-                       unsigned int flags ATTRIBUTE_UNUSED) {
+                       unsigned int flags)
+{
     virStorageDriverStatePtr driver = obj->conn->storagePrivateData;
     virStoragePoolObjPtr pool;
     virStorageBackendPtr backend;
     virStorageVolDefPtr voldef = NULL;
     virStorageVolPtr ret = NULL, volobj = NULL;
 
+    virCheckFlags(0, NULL);
+
     storageDriverLock(driver);
     pool = virStoragePoolObjFindByUUID(&driver->pools, obj->uuid);
     storageDriverUnlock(driver);
@@ -1385,7 +1404,8 @@ static virStorageVolPtr
 storageVolumeCreateXMLFrom(virStoragePoolPtr obj,
                            const char *xmldesc,
                            virStorageVolPtr vobj,
-                           unsigned int flags ATTRIBUTE_UNUSED) {
+                           unsigned int flags)
+{
     virStorageDriverStatePtr driver = obj->conn->storagePrivateData;
     virStoragePoolObjPtr pool, origpool = NULL;
     virStorageBackendPtr backend;
@@ -1393,6 +1413,8 @@ storageVolumeCreateXMLFrom(virStoragePoolPtr obj,
     virStorageVolPtr ret = NULL, volobj = NULL;
     int buildret;
 
+    virCheckFlags(0, NULL);
+
     storageDriverLock(driver);
     pool = virStoragePoolObjFindByUUID(&driver->pools, obj->uuid);
     if (pool && STRNEQ(obj->name, vobj->pool)) {
@@ -1464,7 +1486,7 @@ storageVolumeCreateXMLFrom(virStoragePoolPtr obj,
     }
 
     if (origvol->building) {
-        virStorageReportError(VIR_ERR_INTERNAL_ERROR,
+        virStorageReportError(VIR_ERR_OPERATION_INVALID,
                               _("volume '%s' is still being allocated."),
                               origvol->name);
         goto cleanup;
@@ -1592,7 +1614,7 @@ storageVolumeDownload(virStorageVolPtr obj,
     if (virFDStreamOpenFile(stream,
                             vol->target.path,
                             offset, length,
-                            O_RDONLY, false) < 0)
+                            O_RDONLY) < 0)
         goto out;
 
     ret = 0;
@@ -1645,7 +1667,7 @@ storageVolumeUpload(virStorageVolPtr obj,
     }
 
     if (vol->building) {
-        virStorageReportError(VIR_ERR_INTERNAL_ERROR,
+        virStorageReportError(VIR_ERR_OPERATION_INVALID,
                               _("volume '%s' is still being allocated."),
                               vol->name);
         goto out;
@@ -1656,7 +1678,7 @@ storageVolumeUpload(virStorageVolPtr obj,
     if (virFDStreamOpenFile(stream,
                             vol->target.path,
                             offset, length,
-                            O_WRONLY, false) < 0)
+                            O_WRONLY) < 0)
         goto out;
 
     ret = 0;
@@ -1854,7 +1876,7 @@ storageVolumeWipe(virStorageVolPtr obj,
     }
 
     if (vol->building) {
-        virStorageReportError(VIR_ERR_INTERNAL_ERROR,
+        virStorageReportError(VIR_ERR_OPERATION_INVALID,
                               _("volume '%s' is still being allocated."),
                               vol->name);
         goto out;
@@ -1914,7 +1936,7 @@ storageVolumeDelete(virStorageVolPtr obj,
     }
 
     if (vol->building) {
-        virStorageReportError(VIR_ERR_INTERNAL_ERROR,
+        virStorageReportError(VIR_ERR_OPERATION_INVALID,
                               _("volume '%s' is still being allocated."),
                               vol->name);
         goto cleanup;
@@ -2012,13 +2034,16 @@ cleanup:
 
 static char *
 storageVolumeGetXMLDesc(virStorageVolPtr obj,
-                        unsigned int flags ATTRIBUTE_UNUSED) {
+                        unsigned int flags)
+{
     virStorageDriverStatePtr driver = obj->conn->storagePrivateData;
     virStoragePoolObjPtr pool;
     virStorageBackendPtr backend;
     virStorageVolDefPtr vol;
     char *ret = NULL;
 
+    virCheckFlags(0, NULL);
+
     storageDriverLock(driver);
     pool = virStoragePoolObjFindByName(&driver->pools, obj->pool);
     storageDriverUnlock(driver);
